//
//  NSMutableDictionary+ImageMetadata.m
//  NSDictionary-ImageMetadata
//
//  Created by Steven Grace on 4/22/13.
//  Copyright (c) 2013 Steven Grace. All rights reserved.
//

#import "NSMutableDictionary+ImageMetadata.h"

@interface NSDictionary (Private_Ext)

- (id)recursiveObjectForKey:(NSString *)key;
@end

@implementation NSMutableDictionary (ImageMetadata)


#pragma mark Metadata Fetching
//
//
//
+(NSMutableDictionary*)metadataWithImageAtURL:(NSURL*)url{
    
    
    CGImageSourceRef imageSource = CGImageSourceCreateWithURL((__bridge CFURLRef)url, NULL);
    
    if(!imageSource){
        NSLog(@"%s failed to create image at url: %@",__PRETTY_FUNCTION__,url);
        return nil;
    }
    NSDictionary *options = [NSDictionary dictionaryWithObjectsAndKeys:
                             [NSNumber numberWithBool:NO], (NSString *)kCGImageSourceShouldCache,
                             nil];
    
    CFDictionaryRef imageProperties =CGImageSourceCopyPropertiesAtIndex(imageSource,0,(__bridge CFDictionaryRef)options);
    
    CFRelease(imageSource);
    
    NSMutableDictionary* metadata =
    [NSMutableDictionary dictionaryWithDictionary:(__bridge NSDictionary*)imageProperties];
    
    CFRelease(imageProperties);
    
    return metadata;
    
}
#pragma mark - Metadata Saving
//
//
//
+(BOOL)writeMetadata:(NSDictionary*)metadata toImageAtURL:(NSURL*)url{
 
    CGImageSourceRef source=CGImageSourceCreateWithURL((__bridge CFURLRef)url, NULL);
    
    CFStringRef UTI = CGImageSourceGetType(source);
    
    NSMutableData *dest_data = [NSMutableData dataWithContentsOfURL:url];
    
    CGImageDestinationRef destination = CGImageDestinationCreateWithData((__bridge CFMutableDataRef)(dest_data),UTI,1,NULL);
    
    CGImageDestinationAddImageFromSource(destination,source,0,(__bridge CFDictionaryRef)(metadata));
    //tell the destination to write the image data and metadata into our data object.
    //It will return false if something goes wrong
    BOOL success = CGImageDestinationFinalize(destination);
    if(!success)
        NSLog(@"%s:Error setting Metadata to image file",__PRETTY_FUNCTION__);
    
    CFRelease(destination);
    CFRelease(source);
    return success;

}
#pragma mark - Public and Formatted Property Helpers
//
//
//
- (id)valueForProperty:(NSString *)property{
    return [self recursiveObjectForKey:property];
}
//
//
//
-(NSString*)formattedShutterSpeed{
    NSNumber* time = [self recursiveObjectForKey:(id)kCGImagePropertyExifShutterSpeedValue];
    float d=1/[time floatValue];
    return [NSString stringWithFormat:@"1/%g",ceilf(d)];
}
//
//
//
-(NSString*)formattedApetureValue{
    return
    [NSString stringWithFormat:@"f/%@",[self recursiveObjectForKey:(id)kCGImagePropertyExifApertureValue]];
}
#pragma mark - Location
//
//
//
-(CLLocation*)location{
    
    NSDictionary *GPSinfo = [self objectForKey:(id)kCGImagePropertyGPSDictionary];
    if (GPSinfo) {
        
        CLLocationDegrees lat = [[GPSinfo objectForKey:(id)kCGImagePropertyGPSLatitude] floatValue];
        CLLocationDegrees lng = [[GPSinfo objectForKey:(id)kCGImagePropertyGPSLongitude] floatValue];
        NSString *latitudeRef = [GPSinfo objectForKey:(id)kCGImagePropertyGPSLatitudeRef];
        NSString *longtiudeRef = [GPSinfo objectForKey:(id)kCGImagePropertyGPSLongitudeRef];
        
        if ([@"S" isEqualToString:latitudeRef])
            lat *= -1.0f;
                
        if ([@"W" isEqualToString:longtiudeRef])
            lng *= -1.0f;
        
        
        CLLocation *location =
            [[CLLocation alloc] initWithLatitude:lat longitude:lng];
        return location;
    }
    return nil;
}
//
//
//
-(CLLocationDirection)trueHeading {

    NSDictionary *GPSinfo = [self objectForKey:(id)kCGImagePropertyGPSDictionary];
    CLLocationDirection heading = 0;
    if (GPSinfo) {
        heading =[[GPSinfo objectForKey:(id)kCGImagePropertyGPSImgDirection] doubleValue];
    }
    return heading;
}
#pragma mark- Recursive Extenstion
/*
 //
 // Credit and Source for snippet:
 // http://viamike.com/code/recursively-searching-an-nsdictionary-for-a-key
 */
- (id)recursiveObjectForKey:(NSString *)key{
    
    if([self.allKeys containsObject:key]) {
        
        // this dictionary contains the key, return the value
        return [self objectForKey:key];
    }
    
    for(NSString *k in self.allKeys) {
        id obj = [self objectForKey:k];
        if([obj isKindOfClass:[NSDictionary class]]) {
            
            // we found a child dictionary, let's traverse it
            NSDictionary *d = (NSDictionary *)obj;
            id child = [d recursiveObjectForKey:key];
            if(child) return child;
            
        } else if([obj isKindOfClass:[NSArray class]]) {
            
            
            // loop through the NSArray and traverse any dictionaries found
            NSArray *a = (NSArray *)obj;
            for(id child in a) {
                if([child isKindOfClass:[NSDictionary class]]) {
                    NSDictionary *d = (NSDictionary *)child;
                    id o = [d recursiveObjectForKey:key];
                    if(o) return o;
                }
            }
        }
    }
    
    // the key was not found in this dictionary or any of it's children
    return nil;
}
@end
